/*

 parport.h

 Written by
  Kajtar Zsolt <soci@c64.rulez.org>

    This file is part of IDEDOS the IDE64 disk operating system
    See README for copyright notice.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#if defined __linux__ || defined __FreeBSD__ || defined __DJGPP__ || defined _WIN32 || defined __CYGWIN__
#ifndef _PARPORT_H
#define _PARPORT_H

extern int parport_init(const char *, unsigned int);
extern void parport_deinit(void);

#if defined _WIN32 || defined __CYGWIN__
#define PORTTALK
#define INPOUT32
#define PORTIO
static __inline__ void outb(unsigned char _data, unsigned short _port)
{
    __asm__ __volatile__ ("outb %1, %0" : : "dN" (_port), "a" (_data));
}

static __inline__ unsigned char inb(unsigned short _port)
{
    unsigned char rv;
    __asm__ __volatile__ ("inb %1, %0" : "=a" (rv) : "dN" (_port));
    return rv;
}

#ifdef INPOUT32
extern int parport_libinit(void);
extern void parport_libdeinit(void);

typedef short (__stdcall *inpfuncPtr)(short portaddr);
typedef void (__stdcall *oupfuncPtr)(short portaddr, short datum);

extern inpfuncPtr inp32;
extern oupfuncPtr oup32;
#endif

#elif defined __FreeBSD__
#define PARPORT

#if defined __x86_64__ || defined __i386__
#define PORTIO
static __inline unsigned char inb(unsigned short port) {
    unsigned char value;
    __asm __volatile("inb %%dx,%0" : "=a" (value) : "d" (port));
    return value;
}

static __inline void outb(unsigned char value, unsigned short port) {
    __asm __volatile("outb %0,%1" : : "a" (value), "id" (port));
}
#endif

#ifdef PARPORT
#include <dev/ppbus/ppi.h>
#include <dev/ppbus/ppbconf.h>
#include <sys/ioctl.h>
#define PARPORT_RSTATUS PPIGSTATUS
#define PARPORT_RCONTROL PPIGCTRL
#define PARPORT_WCONTROL PPISCTRL
#define PARPORT_RDATA PPIGDATA
#define PARPORT_WDATA PPISDATA
#define PARPORT_STATUS_BUSY nBUSY
#define PARPORT_STATUS_ERROR nFAULT
#define parport(a, b) ioctl(parportfd, a, b)

extern int parportfd;
#endif
#elif defined __DJGPP__
#define PORTIO
static inline unsigned char inb(unsigned short _port)
{
    unsigned char rv;
    __asm__ __volatile__ ("inb %1, %0" : "=a" (rv) : "dN" (_port));
    return rv;
}

static inline void outb(unsigned char _data, unsigned short _port)
{
    __asm__ __volatile__ ("outb %1, %0" : : "dN" (_port), "a" (_data));
}
#else /* Linux */
#define PARPORT

#if defined __x86_64__ || defined __i386__
#define PORTIO
#include <sys/io.h>
#endif

#ifdef PARPORT
#include <linux/ppdev.h>
#include <linux/parport.h>
#include <sys/ioctl.h>
#define PARPORT_RSTATUS PPRSTATUS
#define PARPORT_RCONTROL PPRCONTROL
#define PARPORT_WCONTROL PPWCONTROL
#define PARPORT_RDATA PPRDATA
#define PARPORT_WDATA PPWDATA
#define parport(a, b) ioctl(parportfd, a, b)

extern int parportfd;
#endif

#endif

#endif
#endif
